%% Script to simulate data from a logit

clc; clear all; 

global fctCount


% --- SETUP --- 
mp.K = 10;          % # of characteristics
mp.J = 10;         % # of products 
mp.D = 1;           % # of demographics
mp.N = 20000;       % # of individuals
mp.lambda = 6;      % scale of the logit error

% scale
mp.myScale = 1; % must be positive integer

mp.b = rand(mp.D,mp.K).*repmat(1:mp.myScale:mp.myScale*mp.K,mp.D,1); 
data.x = randn(mp.K,mp.J); 
data.z = randn(mp.N,mp.D); 

% utilities
v = data.z*mp.b*data.x/mp.lambda;% + 10*randn(mp.N,mp.J); 
logsum = log(sum(exp(v),2)); 

% CCPs
ccps = exp(v - logsum(:,ones(1,mp.J))); 

% Simulating
unis = rand(mp.N,1); 
data.choice = sum(unis(:,ones(1,mp.J))>cumsum(ccps,2),2)+1;
data.choiceIdx = sub2ind(size(v),1:mp.N,data.choice'); 
% Printing
%tabulate(data.choice); 

% Likelihood
%mean(ll_rclogit(b,choiceIdx,x,z)) 

% cleanup
clear v logsum ccps unis

% Testing
%addpath('./DERIVESTsuite'); 
%[deriv,err] = hessian(ll,mp.b); 

%% for saving
thetahats = nan(numel(mp.b),3); % 3 = num, anal, black
llvals = nan(1,3); 

%% Optimization

% Setting up anonymous function and optimization stuff
modes = { 'Numeric' , 'Analytic' , 'BlackBox' }; 
i = 2; 
 
fprintf('--- Chosen mode: %s --- \n',char(modes(i))); 

if i == 1; 
    ll = @(theta) - mean(ll_rclogit(theta,data,mp)); 
    opt = optimoptions(@fminunc,'Display','Iter','Algorithm','quasi-newton','HessUpdate','bfgs'); 
elseif i == 2 
    ll = @(theta) ll_rclogit_anal(theta,data,mp); 
    % BHHH
    opt = optimoptions(@fminunc,'Display','Iter','Algorithm','trust-region','GradObj','On','Hessian','On','DerivativeCheck','Off'); 
    % BFGS @ analytic gradients
    %opt = optimoptions(@fminunc,'Display','Iter','Algorithm','trust-region','GradObj','On','HessUpdate','bfgs'); 
elseif i == 3
    ll = @(theta) ll_rclogit_black(theta,data,mp); 
    % BHHH 
    %opt = optimoptions(@fminunc,'Display','Iter','Algorithm','trust-region','GradObj','On','Hessian','On'); 
    % BHHH Quasi newton
    opt = optimoptions(@fminunc,'Display','Iter','Algorithm','quasi-newton','GradObj','On','Hessian','On'); 
    % BFGS (trust-region)
    %opt = optimoptions(@fminunc,'Display','Iter','Algorithm','trust-region','GradObj','On','Hessian','Off','HessUpdate','bfgs'); 
    % BFGS, quasi-newton
    %opt = optimoptions(@fminunc,'Display','Iter','Algorithm','quasi-newton','GradObj','On','Hessian','Off','HessUpdate','bfgs'); 
end;



% Optimization
scaleTheta0 = 0.85; 
theta0 = (1 + scaleTheta0*(rand(1,10)-0.5)).*mp.b; 

fctCount = 0;
t = tic; 
[thetahat,ll_thetahat,flag,output] = fminunc(ll,theta0,opt); 

fprintf('Iteration count = %i.\n',fctCount); 
fprintf('Time to convergence = %5.2f sec.\n',toc(t)); 

fprintf('Mean rel. difference from truth = %5.3f.\n',mean((thetahat-mp.b)./mp.b)); 
fprintf('||thetahat-theta0||/theta0 = %5.3f.\n',norm(thetahat-mp.b)/norm(mp.b)); 

thetahats(:,i) = thetahat;
llvals(i) = ll_thetahat; 

%% Printing
fprintf('__________________________________________\n'); 
fprintf('Results\n'); 
fprintf('\t True \t Num \t Anal \t Black\n'); 
fprintf('__________________________________________\n'); 
for i=1:size(thetahats,1); 
    fprintf('%i\t %5.3f \t %5.3f \t %5.3f \t %5.3f\n',i,mp.b(i),thetahats(i,1),thetahats(i,2),thetahats(i,3)); 
end; 
fprintf('__________________________________________\n\n'); 
fprintf('like-mean  %5.3g %5.3g %5.3g \n',llvals(1)-mean(llvals),llvals(2)-mean(llvals),llvals(3)-mean(llvals)); 

%% Looking at the first parameter
h = 1e-8; 
k = 2 * 5; 
vals = mp.b(1) + h *(-floor(k/2)-.5:floor(k/2)+.5); 
fvals = nan(1,k+2); 
fdvals = nan(numel(mp.b),k+2); 
fdvals_black = nan(numel(mp.b),k+2); 
for i=1:k+2; 
    [fvals(i),fdvals(:,i)] = ll_rclogit_anal([vals(i),mp.b(2:end)],data,mp);
    % get the finite difference used by the Black-Box function
    [~,fdvals_black(:,i)] = ll_rclogit_black([vals(i),mp.b(2:end)],data,mp);
end; 
fdvals = fdvals(1,:); 
fdvals_black = fdvals_black(1,:); 
%plot(vals,fvals,'ok'); 
%plot(vals(1:end-1),diff(fvals)./h,'ok',vals(1:end-1),fdvals(1:end-1),'-b',vals(1:end-1),fdvals_black(1:end-1),'xr'); 
[fdvals_black; fdvals] - mean(fdvals)
plot(vals(1:end-1),fdvals(1:end-1)-mean(fdvals),'-b',vals(1:end-1),fdvals_black(1:end-1)-mean(fdvals),'xr'); 

%% Noise estimation

fprintf('--- Noise estimation ---\n'); 
fprintf('Input function: %s.\n',char(ll)); 

func = @(x) ll(x); 

% Direction vector (random) 
p = rand(mp.D,mp.K); 
p = p/norm(p); % rescale to a step of 2-norm=1

% Define the number of additional evaluations
m = 14;

% Define the sampling distance
h = 1e-8;

% Evaluate the function at m+1 equally spaced points  
fval = zeros(m+1,1);
mid = floor((m+2)/2);
for i=1:m+1
    s = 2*(i-mid)/m;
    x = mp.b + s*h*p;
    fval(i) = feval(func,x);
end

% Call the noise estimator
[fnoise,level,inform] = ECNoise(m+1,fval)

rel_noise = fnoise/fval(mid)
